<?php

class CombineCSS {

	/**
	*Variables
	*/
	const nspace = 'combine-css';
	const pname = 'Combine CSS';
	const version = 0.8;

	protected $_plugin_file;
	protected $_plugin_dir;
	protected $_plugin_path;
	protected $_plugin_url;

    var $doc_root = '';
	var $cachetime = '';
	var $create_cache = false;
	var $upload_path = '';
	var $upload_uri = '';
	var $tmp_dir = '';

	var $css_domain = '';
	var $css_path = '';
	var $css_uri = '';
	var $css_path_tmp = '';
	var $css_token = '';
	var $css_settings_path = '';

	var $settings_fields = array();
	var $settings_data = array();
	var $css_files_ignore = array( 'admin-bar.css', 'login.css', 'colors-fresh.css', 'wp-admin.css' );
	var $css_handles_found = array();
	var $debug = false;

	/**
	*Constructor
	*
	*@return void
	*@since 0.1
	*/
	function __construct() {}

	/**
	*Init function
	*
	*@return void
	*@since 0.1
	*/
	function init() {

        // set doc root

        $this->doc_root = ABSPATH;

		// internationalize

		add_action( 'init', array( &$this, 'internationalize' ) );

		// add delete css cache button

		add_action( 'wp_before_admin_bar_render', array( &$this, 'delete_cache_button' ) );

        // settings fields

        $this->settings_fields = array(
                        'legend_1' => array(
                                'label' => 'General Settings',
                                'type' => 'legend'
                                ),
                        'css_domain' => array(
                                'label' => 'CSS Domain',
                                'type' => 'text',
                                'default' => get_option( 'siteurl' )
                                ),
                        'cachetime' => array(
                                'label' => 'Cache Expiration',
                                'instruction' => 'How often to refresh CSS files in seconds.',
                                'type' => 'select',
                                'values' => array( '60' => '1 minute', '300' => '5 minutes', '900' => '15 minutes', '1800' => '30 minutes', '3600' => '1 hour' ),
                                'default' => '300'
                                ),
                        'htaccess_user_pw' => array(
                                'label' => 'Username and Password',
                                'instruction' => 'Use when site is accessed behind htaccess authentication -- syntax: username:password.',
                                'type' => 'text',
                                'default' => 'username:password'
                                ),
                        'add_gf_css' => array(
                                'label' => 'Add Gravity Forms CSS',
                                'type' => 'select',
                                'values' => array( 'No' => 'No', 'Yes' => 'Yes' ),
                                'default' => 'No'
                                ),
                        'ignore_files' => array(
                                'label' => 'CSS Files to Ignore',
                                'instruction' => 'Enter one per line. Only use the name of the CSS file (like style.css). You can be more specific about what CSS file to ignore by specifying the plugin and then the CSS file (like plugin:style.css).',
                                'type' => 'textarea'
                                ),
                        'compress' => array(
                                'label' => 'GZip Compress CSS output?',
                                'type' => 'select',
                                'values' => array( 'No' => 'No', 'Yes' => 'Yes' ),
                                'default' => 'Yes'
                                ),
                        'debug' => array(
                                'label' => 'Turn on debugging?',
                                'type' => 'select',
                                'values' => array( 'No' => 'No', 'Yes' => 'Yes' ),
                                'default' => 'No'
                                )
                    );

		// settings data

		$this->settings_data = unserialize( get_option( self::nspace . '-settings' ) );
        if ( ! $this->settings_data ) $this->settings_data = array();
        foreach ( $this->settings_fields as $key => $val ) {
            if ( ! isset( $this->settings_data[$key] ) ) $this->settings_data[$key] = '';
        }
		$this->cachetime = $this->get_settings_value( 'cachetime' );
		if ( ! @strlen( $this->cachetime ) ) $this->cachetime = 300;
		$this->css_domain = $this->get_settings_value( 'css_domain' );
		if ( ! @strlen( $this->css_domain ) ) $this->css_domain = get_option( 'siteurl' );
		if ( $this->settings_data['debug'] == 'Yes' ) $this->debug = true;
		if ( ! $this->settings_data['compress'] ) $this->settings_data['compress'] = 'Yes';

		// add ignore files

		$ignore_list = explode( "\n", $this->settings_data['ignore_files'] );
		foreach ( $ignore_list as $item ) $this->css_files_ignore[] = $item;

		// check upload dirs

		$this->check_upload_dirs();

		if ( is_admin() ) {

			// add admin settings page

			add_action( 'admin_menu', array( &$this, 'add_settings_page' ), 30 );
		}
		elseif ( strstr( $_SERVER['REQUEST_URI'], 'wp-login' ) || strstr( $_SERVER['REQUEST_URI'], 'gf_page=' ) || strstr( $_SERVER['REQUEST_URI'], 'preview=' ) ) {}
		elseif ( ! file_exists( $this->css_settings_path ) ) {}
		else {

			// gather and combine css function calls

			add_action( 'style_loader_tag', array( $this, 'gather_css' ), 500 );
			add_filter( 'wp_head', array( $this, 'combine_css' ), 501 );

			// get rid of browser prefetching of next page from link rel="next" tag

			remove_action( 'wp_head', 'adjacent_posts_rel_link_wp_head' );
			remove_action( 'wp_head', 'adjacent_posts_rel_link' );
		}

		// if delete css button is click, delete cache

		if ( isset( $_GET[ 'action' ] ) && $_GET[ 'action' ] == 'deletecsscache' && isset( $_GET[ '_wpnonce' ] ) ) {
			add_action( 'admin_init', array( &$this, 'admin_bar_delete_cache' ) );
		}
	}

	/**
	*Check upload dirs
	*
	*@return void
	*@since 0.4
	*/
	function check_upload_dirs() {

		// make sure upload dirs exist and set file path and uri

		$upload_dir = wp_upload_dir();
		if ( ! file_exists( $upload_dir['basedir'] ) ) mkdir ( $upload_dir['basedir'] );
		$this->upload_path = $upload_dir['basedir'] . '/' . self::nspace . '/';
		$this->upload_uri = $upload_dir['baseurl'] . '/' . self::nspace . '/';
		if ( ! file_exists( $this->upload_path ) ) mkdir ( $this->upload_path );

		// create tmp directory

		$this->create_tmp_dir();

		// write settings to temp file so that css script has access to WP settings without having to load WP infrastructure

		$this->css_settings_path = $this->tmp_dir . $_SERVER['HTTP_HOST'] . '-settings.dat';
		if ( $this->cache_expired( $this->css_settings_path, false ) ) {
			$args = array( 'upload_path' => $this->upload_path, 'compress' => $this->settings_data['compress'] );
			$this->write_file( $this->css_settings_path, serialize( $args ) );
		}
	}

	/**
	*Create tmp dir
	*
	*@return void
	*@since 0.4
	*/
	function create_tmp_dir() {
		$this->tmp_dir = $this->get_plugin_path() . '/tmp/';
		if ( ! file_exists( $this->tmp_dir ) && is_writable( $this->tmp_dir ) ) mkdir ( $this->tmp_dir );
	}

	/**
	*Translation
	*
	*@return void
	*@since 0.1
	*/
	function internationalize() {
		load_plugin_textdomain( self::nspace, false, $this->get_plugin_dir() . '/lang' );
	}

	/**
	*Debug function
	*
	*@return void
	*@since 0.1
	*/
	function debug ( $msg ) {
		if ( $this->debug ) error_log( 'DEBUG: ' . $msg );
	}

	/**
	*Cache expired?
	*
	*@return boolean
	*@since 0.1
	*/
	function cache_expired ( $path, $debug = true ) {
		$mtime = 0;
		if( file_exists( $path ) && filesize( $path ) ) $mtime = @filemtime( $path );
		if ( ( time() - $mtime ) > $this->cachetime ) {
            if ( $debug ) $this->debug( 'CACHE EXPIRED (' . $path . ')' );
            if ( $debug ) $this->debug( 'TIME SINCE (' . ( time() - $mtime ) . ') and Cache Time (' . $this->cachetime . ')' );
            return true;
        }
        if ( $debug ) $this->debug( 'USING CACHE (' . $path . ')' );
		return false;
	}

	/**
	*Gather CSS
	*
	*@return string
	*@since 0.1
	*/
	function gather_css ( $src ) {

		$this->debug( 'Function gather_css' );

		// get src of CSS file

		preg_match( "/<link.*href\=('|\")(http.*(\/wp\-(content|includes|admin).*))(\?.*)('|\") type.*$/", $src, $matches );
        if ( ! $matches ) return $src;
		//$css_src = $this->strip_domain( $matches[2] );
        $css_src = $matches[3];
		$css_file = $this->get_file_from_src( $css_src );

		// get context (plugin or theme)

		preg_match( "/(plugins|themes)\/(.*)\/.*/", $css_src, $cmatches );
		$context = '';
		if ( $cmatches ) {
			$context = $cmatches[2];
			if ( strstr( $context, '/' ) ) $context = dirname( $cmatches[2] );
		}
        if ( $context ) $this->debug( 'Context (' . $context . ')' );

		// add this css file to the found handles if it is not to be ignored

		if ( ! in_array( $context . ':' . $css_file, $this->css_files_ignore ) && ! in_array( $css_file, $this->css_files_ignore ) ) {

			// save all the scripts that we've gathered

			$this->debug( 'CSS found (' . $css_src . ')' );
			$this->css_handles_found[$css_src]['css_file'] = $css_file;
			$this->css_handles_found[$css_src]['css_src'] = $matches[2] . $matches[5];
			$this->css_handles_found[$css_src]['context'] = $context;
		}

		// get name of file (token) based on md5 hash of css handles

		$this->css_token = md5( @implode( '', array_keys( $this->css_handles_found ) ) );

		// add gravity forms css, if told to

		if ( $this->settings_data['add_gf_css'] == 'Yes' ) {
			$gf_src = $this->strip_domain( plugins_url( 'css/forms.css', 'gravityforms' ) );
			$gf_file = $this->get_file_from_src( $gf_src );
			$this->css_handles_found[$gf_src]['css_file'] = $gf_file;
		}

		// paths to css

		$this->css_path = $this->upload_path . $this->css_token . '.css';
		$this->css_uri = $this->upload_uri . $this->css_token . '.css';
		$this->css_path_tmp = $this->css_path . '.tmp';

        if ( $this->cache_expired( $this->css_path ) && $this->cache_expired( $this->css_path_tmp ) )  {
            $this->create_cache = true;
        }

		// return src if it's not found in handles

		if ( isset( $this->css_handles_found[$css_src] ) ) {}
		else return $src;
	}

	/**
	*Combine CSS
	*
	*@return void
	*@since 0.1
	*/
	function combine_css ( ) {
		$this->debug( 'Function combine_css' );
		$content = '';
        if ( $this->create_cache && $this->cache_expired( $this->css_path ) ) {

			// loop through css handles

			foreach ( $this->css_handles_found as $css_src => $array ) {
				$css_file = $array['css_file'];
				if( $this->file_exists( $css_src ) ) {
					$css_content = '';

					// if file is a PHP script, pull content via curl

					if ( preg_match( "/\.php/", $css_src ) ) {
						$css_content = $this->curl_file_get_contents ( $this->css_handles_found[$css_src]['css_src'] );
						$css_src = $this->css_handles_found[$css_src]['css_src'];
					}
					else $css_content = file_get_contents( $this->doc_root . $css_src );

					// change path to images

					foreach ( array( 'images', 'fonts', 'assets' ) as $folder ) {
					$css_content = str_replace(
									array( 
										'url(' . $folder . '/',
										"url('" . $folder . "/",
										'url("' . $folder . '/',
										'url(../' . $folder . '/',
										"url('../" . $folder . "/",
										'url("../' . $folder . '/'
									),
									array( 
										'url(' . dirname( $css_src ) . '/' . $folder . '/',
										'url(\'' . dirname( $css_src ) . '/' . $folder . '/',
										'url("' . dirname( $css_src ) . '/' . $folder . '/',
										'url(' . dirname( dirname( $css_src ) ) . '/' . $folder . '/',
										'url(\'' . dirname( dirname( $css_src ) ) . '/' . $folder . '/',
										'url("' . dirname( dirname( $css_src ) ) . '/' . $folder . '/'
									), 
									$css_content
								);
					}
					$content .= $this->compress( $css_content );
					unset( $css_content );
				}
			}
			$this->cache( $content );
            $this->debug( 'Create Combined File (' . $this->css_path . ')' );
			@rename( $this->css_path_tmp, $this->css_path );
		}
		if ( file_exists ( $this->css_path ) ) {
			echo "\t\t<link rel='stylesheet' id='combine-css' href='" . str_replace( get_option( 'siteurl' ), $this->css_domain, $this->get_plugin_url() . 'css.php?token=' . $this->css_token . '&#038;ver=' . self::version ) . "' type='text/css' media='all' />\n";
		}
	}

	/**
	*File exists
	*
	*@return boolean
	*@since 0.1
	*/
	function file_exists( $src ) {
		if ( @strlen( $src ) && file_exists( $this->doc_root . $src ) ) return true;
		return false;
	}

	/**
	*Get file from src
	*
	*@return string
	*@since 0.1
	*/
	function get_file_from_src( $src ) {
		$frags = explode( '/', $src );
		return $frags[count( $frags ) - 1];
	}

	/**
	*cache data
	*
	*@return void
	*@since 0.1
	*/
	function cache( $content ) {
		if ( ! file_exists( $this->css_path_tmp ) ) $this->write_file( $this->css_path_tmp, $content );
	}

	/**
	*Write file
	*
	*@return void
	*@since 0.4
	*/
	function write_file ( $file, $content ) {
        if ( is_writable ( dirname( $file ) ) ) {
            $this->debug( 'write_file (' . $file . ')' );
            $fp = fopen( $file, "w" );
            if ( flock( $fp, LOCK_EX ) ) { // do an exclusive lock
                fwrite( $fp, $content );
                flock( $fp, LOCK_UN ); // release the lock
            }
            fclose( $fp );
        }
	}

	/**
	*Get content via curl
	*
	*@return string
	*@since 0.1
	*/
	function curl_file_get_contents( $src ) {
		$url = trim( $src );
		$url = preg_replace( "/http(|s):\/\//", "http://" . $this->get_settings_value( 'htaccess_user_pw' ) . "@", $url );
		$c = curl_init();
		curl_setopt( $c, CURLOPT_URL, $url );
		curl_setopt( $c, CURLOPT_FAILONERROR, false );
		curl_setopt( $c, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $c, CURLOPT_VERBOSE, false );
		curl_setopt( $c, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $c, CURLOPT_SSL_VERIFYHOST, false );
		if( count( $header ) ) curl_setopt( $c, CURLOPT_HTTPHEADER, $header );
		$contents = curl_exec( $c );
		curl_close( $c );
		return $contents;
	}

	/**
	*Strip domain from string
	*
	*@return string
	*@since 0.1
	*/
	function strip_domain( $src ) {
		$src = str_replace( array( 'http://', 'https://' ), array( '', '' ), $src );
		$frags = explode( '/', $src );
		array_shift( $frags );
		return implode( '/', $frags );
	}

	/**
	*Compress 
	*
	*@return string
	*@since 0.1
	*/
	function compress( $content ) {
		return $this->minify( $content );
	}

	/**
	*Minify
	*
	*@return string
	*@since 0.2
	*/
	function minify( $content ) {
		$items = array(
				'#/\*.*?\*/#s' => '',
				'/(\t|\r|\n)/' => '',
				'/[^}{]+{\s?}/' => '',
				'/\s*{\s*/' => '{',
				'/;}/' => '}',
				'/, /' => ',',
				'/; /' => ';',
				'/: /' => ':',
				'/\s+/' => ' ',
				);
		foreach( $items as $search => $replace ) $content = preg_replace( $search, $replace, $content );
		return $content;
	}

	/**
	*Add settings page
	*
	*@return void
	*@since 0.1
	*/
	function add_settings_page() {
		if ( current_user_can( 'manage_options' ) ) add_options_page( self::pname, self::pname, 'manage_options', 'combine-css-settings', array( &$this, 'settings_page' ) );
	}

	/**
	*Settings page
	*
	*@return void
	*@since 0.1
	*/
	function settings_page() {
		if ( isset( $_POST['combine-css_update_settings'] ) ) $this->update_settings();
		$this->show_settings_form();
	}

	/**
	*Show settings form
	*
	*@return void
	*@since 0.1
	*/
	function show_settings_form () {
		include( $this->get_plugin_path() . '/views/admin_settings_form.php' );
	}

	/**
	*Get single value from unserialized data
	*
	*@return string
	*@since 0.1
	*/
	function get_settings_value( $key = '' ) {
        if ( isset( $this->settings_data[$key] ) ) return $this->settings_data[$key];
	}

	/**
	*Remove option when plugin is deactivated
	*
	*@return void
	*@since 0.1
	*/
	function delete_settings() {
		delete_option( $this->option_key );
	}

	/**
	*Is associative array function
	*
	*@return string
	*@since 0.1
	*/
	function is_assoc( $arr ) {
		if ( isset ( $arr[0] ) ) return false;
		return true;
	}

	/**
	*Display a select form element
	*
	*@return string
	*@since 0.1
	*/
	function select_field( $name, $values, $value, $use_label = false, $default_value = '', $custom_label = '' ) {
		ob_start();
		$label = '-- please make a selection --';
		if ( @strlen( $custom_label ) ) {
			$label = $custom_label;
		}

		// convert indexed array into associative

		if ( ! $this->is_assoc( $values ) ) {
			$tmp_values = $values;
			$values = array();
			foreach ( $tmp_values as $tmp_value ) {
				$values[$tmp_value] = $tmp_value;
			}
		}
?>
	<select name="<?php echo $name; ?>" id="<?php echo $name; ?>">
		<?php if ( $use_label ): ?>
		<option value=""><?php echo $label; ?></option>

		<?php endif; ?>
		<?php foreach ( $values as $val => $label ) : ?>
			<option value="<?php echo $val; ?>"<?php if ($value == $val || ( $default_value == $val && @strlen( $default_value ) && ! @strlen( $value ) ) ) : ?> selected="selected"<?php endif; ?>><?php echo $label; ?></option>
		<?php endforeach; ?>
	</select>
<?php
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	/**
	*Update settings form
	*
	*@return void
	*@since 0.1
	*/
	function update_settings() {
		$data = array();
		foreach( $this->settings_fields as $key => $val ) if( $val['type'] != 'legend' ) $data[$key] = $_POST[$key];
		$this->set_settings( $data );
		$this->delete_cache();
		$this->check_upload_dirs();
	}

	/**
	*Update serialized array option
	*
	*@return void
	*@since 0.1
	*/
	function set_settings( $data ) {
		update_option( self::nspace . '-settings', serialize( $data ) );
		$this->settings_data = $data;
	}

	/**
	*Delete cache
	*
	*@return void
	*@since 0.1
	*/
	function delete_cache() {
		$files = glob( $this->upload_path . '*.css' );
		if ( is_array( $files ) ) array_map( "unlink", $files );
		@unlink( $this->css_settings_path );
		if ( function_exists( 'wp_cache_clear_cache' ) ) wp_cache_clear_cache();
	}

	/**
	*Set plugin file
	*
	*@return void
	*@since 0.1
	*/
	function set_plugin_file( $plugin_file ) {
		$this->_plugin_file = $plugin_file;
	}

	/**
	*Get plugin file
	*
	*@return string
	*@since 0.1
	*/
	function get_plugin_file() {
		return $this->_plugin_file;
	}

	/**
	*Set plugin directory
	*
	*@return void
	*@since 0.1
	*/
	function set_plugin_dir( $plugin_dir ) {
		$this->_plugin_dir = $plugin_dir;
	}

	/**
	*Get plugin directory
	*
	*@return string
	*@since 0.1
	*/
	function get_plugin_dir() {
		return $this->_plugin_dir;
	}

	/**
	*Set plugin file path
	*
	*@return void
	*@since 0.1
	*/
	function set_plugin_path( $plugin_path ) {
		$this->_plugin_path = $plugin_path;
	}

	/**
	*Get plugin file path
	*
	*@return string
	*@since 0.1
	*/
	function get_plugin_path() {
		return $this->_plugin_path;
	}

	/**
	*Set plugin URL
	*
	*@return void
	*@since 0.1
	*/
	function set_plugin_url( $plugin_url ) {
		$this->_plugin_url = $plugin_url;
	}

	/**
	*Get plugin URL
	*
	*@return string
	*@since 0.1
	*/
	function get_plugin_url() {
		return $this->_plugin_url;
	}

	/**
	*Delete cache button
	*
	*@return void
	*@since 0.8
	*/
	function delete_cache_button() {
    	global $wp_admin_bar;
    	if ( ! is_user_logged_in() ) return false;
    	if ( function_exists( 'current_user_can' ) && false == current_user_can( 'delete_others_posts' ) ) return false;
    	$wp_admin_bar->add_menu(
    							array(
						                'parent' => '',
						                'id' => 'delete-css-cache',
						                'title' => __( 'Delete CSS Cache', self::nspace ),
						                'meta' => array( 'title' => __( 'Delete CSS Cache', self::nspace ) ),
						                'href' => wp_nonce_url( admin_url( 'index.php?action=deletecsscache&path=' . urlencode( $_SERVER[ 'REQUEST_URI' ] ) ), 'delete-css-cache' )
						            	) 
    							);
	}

	/**
	*Admin bar delete cache
	*
	*@return void
	*@since 0.8
	*/
	function admin_bar_delete_cache() {
	    if ( function_exists( 'current_user_can' ) && false == current_user_can( 'delete_others_posts' ) ) return false;
    	if ( wp_verify_nonce( $_REQUEST[ '_wpnonce' ], 'delete-css-cache' ) ) {
        	$this->delete_cache();
        	wp_redirect( preg_replace( '/[ <>\'\"\r\n\t\(\)]/', '', $_GET[ 'path' ] ) );
        	die();
    	}
	}
}

?>
